// Copyright 2019 The Bazel Authors. All rights reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.google.devtools.build.lib.network;

/**
 * Interface that gives services (such as the BES or remote build) exposure to their corresponding
 * {@link ConnectivityStatus}.
 */
public interface ConnectivityStatusProvider {
  /** Returns the current connectivity status for a given service. */
  ConnectivityStatus getStatus(String service);
}
