// Copyright 2022 The Bazel Authors. All rights reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.google.devtools.build.importdeps.testdata;

import java.lang.invoke.MethodHandle;
import java.lang.invoke.VarHandle;

public class InvokePolymorphic {
  void f(MethodHandle mh, VarHandle vh) throws Throwable {
    mh.invoke(true, "hello", 42);
    vh.setRelease("foo", 1);
  }
}
